/*
 *  ClientesResource
 *
 * Created on 13 de mayo de 2008, 17:30
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package es.ua.jtech.servcweb.foro.service;

import es.ua.jtech.servcweb.foro.jpa.Cliente;
import java.util.Collection;
import javax.ws.rs.Path;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.ProduceMime;
import javax.ws.rs.ConsumeMime;
import javax.ws.rs.PathParam;
import javax.ws.rs.QueryParam;
import javax.ws.rs.DefaultValue;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.UriInfo;
import es.ua.jtech.servcweb.foro.jpa.Mensaje;
import es.ua.jtech.servcweb.foro.converter.ClientesConverter;
import es.ua.jtech.servcweb.foro.converter.ClienteConverter;


/**
 *
 * @author Miguel Angel
 */

@Path("/clientes/")
public class ClientesResource {
    @Context
    private UriInfo context;
    
    /** Creates a new instance of ClientesResource */
    public ClientesResource() {
    }

    /**
     * Constructor used for instantiating an instance of dynamic resource.
     *
     * @param context HttpContext inherited from the parent resource
     */
    public ClientesResource(UriInfo context) {
        this.context = context;
    }

    /**
     * Get method for retrieving a collection of Cliente instance in XML format.
     *
     * @return an instance of ClientesConverter
     */
    @GET
    @ProduceMime({"application/xml", "application/json"})
    public ClientesConverter get(@QueryParam("start")
    @DefaultValue("0")
    int start, @QueryParam("max")
    @DefaultValue("10")
    int max) {
        try {
            return new ClientesConverter(getEntities(start, max), context.getAbsolutePath());
        } finally {
            PersistenceService.getInstance().close();
        }
    }

    /**
     * Post method for creating an instance of Cliente using XML as the input format.
     *
     * @param data an ClienteConverter entity that is deserialized from an XML stream
     * @return an instance of ClienteConverter
     */
    @POST
    @ConsumeMime({"application/xml", "application/json"})
    public Response post(ClienteConverter data) {
        PersistenceService service = PersistenceService.getInstance();
        try {
            service.beginTx();
            Cliente entity = data.getEntity();
            createEntity(entity);
            service.commitTx();
            return Response.created(context.getAbsolutePath().resolve(entity.getId() + "/")).build();
        } finally {
            service.close();
        }
    }

    /**
     * Returns a dynamic instance of ClienteResource used for entity navigation.
     *
     * @return an instance of ClienteResource
     */
    @Path("{id}/")
    public ClienteResource getClienteResource(@PathParam("id")
    Integer id) {
        return new ClienteResource(id, context);
    }

    /**
     * Returns all the entities associated with this resource.
     *
     * @return a collection of Cliente instances
     */
    protected Collection<Cliente> getEntities(int start, int max) {
        return PersistenceService.getInstance().createQuery("SELECT e FROM Cliente e").setFirstResult(start).setMaxResults(max).getResultList();
    }

    /**
     * Persist the given entity.
     *
     * @param entity the entity to persist
     */
    protected void createEntity(Cliente entity) {
        PersistenceService.getInstance().persistEntity(entity);
        for (Mensaje value : entity.getMensajeCollection()) {
            value.setClienteId(entity);
        }
    }
}
