/*
 *  MensajeResource
 *
 * Created on 13 de mayo de 2008, 17:30
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package es.ua.jtech.servcweb.foro.service;

import es.ua.jtech.servcweb.foro.jpa.Mensaje;
import javax.ws.rs.Path;
import javax.ws.rs.GET;
import javax.ws.rs.PUT;
import javax.ws.rs.DELETE;
import javax.ws.rs.ProduceMime;
import javax.ws.rs.ConsumeMime;
import javax.ws.rs.WebApplicationException;
import javax.persistence.NoResultException;
import es.ua.jtech.servcweb.foro.jpa.Cliente;
import es.ua.jtech.servcweb.foro.converter.MensajeConverter;
import javax.ws.rs.core.UriInfo;


/**
 *
 * @author Miguel Angel
 */

public class MensajeResource {
    private Integer id;
    private UriInfo context;
    
    /** Creates a new instance of MensajeResource */
    public MensajeResource() {
    }

    /**
     * Constructor used for instantiating an instance of dynamic resource.
     *
     * @param context HttpContext inherited from the parent resource
     */
    public MensajeResource(Integer id, UriInfo context) {
        this.id = id;
        this.context = context;
    }

    /**
     * Get method for retrieving an instance of Mensaje identified by id in XML format.
     *
     * @param id identifier for the entity
     * @return an instance of MensajeConverter
     */
    @GET
    @ProduceMime({"application/xml", "application/json"})
    public MensajeConverter get() {
        try {
            return new MensajeConverter(getEntity(), context.getAbsolutePath());
        } finally {
            PersistenceService.getInstance().close();
        }
    }

    /**
     * Put method for updating an instance of Mensaje identified by id using XML as the input format.
     *
     * @param id identifier for the entity
     * @param data an MensajeConverter entity that is deserialized from a XML stream
     */
    @PUT
    @ConsumeMime({"application/xml", "application/json"})
    public void put(MensajeConverter data) {
        PersistenceService service = PersistenceService.getInstance();
        try {
            service.beginTx();
            updateEntity(getEntity(), data.getEntity());
            service.commitTx();
        } finally {
            service.close();
        }
    }

    /**
     * Delete method for deleting an instance of Mensaje identified by id.
     *
     * @param id identifier for the entity
     */
    @DELETE
    public void delete() {
        PersistenceService service = PersistenceService.getInstance();
        try {
            service.beginTx();
            Mensaje entity = getEntity();
            service.removeEntity(entity);
            service.commitTx();
        } finally {
            service.close();
        }
    }

    /**
     * Returns a dynamic instance of ClienteResource used for entity navigation.
     *
     * @param id identifier for the parent entity
     * @return an instance of ClienteResource
     */
    @Path("cliente/")
    public ClienteResource getClienteResource() {
        final Mensaje parent = getEntity();
        return new ClienteResource(null, context) {

            @Override
            protected Cliente getEntity() {
                Cliente entity = parent.getClienteId();
                if (entity == null) {
                    throw new WebApplicationException(new Throwable("Resource for " + context.getAbsolutePath() + " does not exist."), 404);
                }
                return entity;
            }
        };
    }

    /**
     * Returns an instance of Mensaje identified by id.
     *
     * @param id identifier for the entity
     * @return an instance of Mensaje
     */
    protected Mensaje getEntity() {
        try {
            return (Mensaje) PersistenceService.getInstance().createQuery("SELECT e FROM Mensaje e where e.id = :id").setParameter("id", id).getSingleResult();
        } catch (NoResultException ex) {
            throw new WebApplicationException(new Throwable("Resource for " + context.getAbsolutePath() + " does not exist."), 404);
        }
    }

    /**
     * Updates entity using data from newEntity.
     *
     * @param entity the entity to update
     * @param newEntity the entity containing the new data
     * @return the updated entity
     */
    protected Mensaje updateEntity(Mensaje entity, Mensaje newEntity) {
        newEntity.setId(entity.getId());
        entity = PersistenceService.getInstance().mergeEntity(newEntity);
        return entity;
    }
}
