/*
 *  MensajesResource
 *
 * Created on 13 de mayo de 2008, 17:30
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package es.ua.jtech.servcweb.foro.service;

import es.ua.jtech.servcweb.foro.jpa.Mensaje;
import java.util.Collection;
import javax.ws.rs.Path;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.ProduceMime;
import javax.ws.rs.ConsumeMime;
import javax.ws.rs.PathParam;
import javax.ws.rs.QueryParam;
import javax.ws.rs.DefaultValue;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.UriInfo;
import es.ua.jtech.servcweb.foro.converter.MensajesConverter;
import es.ua.jtech.servcweb.foro.converter.MensajeConverter;


/**
 *
 * @author Miguel Angel
 */

@Path("/mensajes/")
public class MensajesResource {
    @Context
    private UriInfo context;
    
    /** Creates a new instance of MensajesResource */
    public MensajesResource() {
    }

    /**
     * Constructor used for instantiating an instance of dynamic resource.
     *
     * @param context HttpContext inherited from the parent resource
     */
    public MensajesResource(UriInfo context) {
        this.context = context;
    }

    /**
     * Get method for retrieving a collection of Mensaje instance in XML format.
     *
     * @return an instance of MensajesConverter
     */
    @GET
    @ProduceMime({"application/xml", "application/json"})
    public MensajesConverter get(@QueryParam("start")
    @DefaultValue("0")
    int start, @QueryParam("max")
    @DefaultValue("10")
    int max) {
        try {
            return new MensajesConverter(getEntities(start, max), context.getAbsolutePath());
        } finally {
            PersistenceService.getInstance().close();
        }
    }

    /**
     * Post method for creating an instance of Mensaje using XML as the input format.
     *
     * @param data an MensajeConverter entity that is deserialized from an XML stream
     * @return an instance of MensajeConverter
     */
    @POST
    @ConsumeMime({"application/xml", "application/json"})
    public Response post(MensajeConverter data) {
        PersistenceService service = PersistenceService.getInstance();
        try {
            service.beginTx();
            Mensaje entity = data.getEntity();
            createEntity(entity);
            service.commitTx();
            return Response.created(context.getAbsolutePath().resolve(entity.getId() + "/")).build();
        } finally {
            service.close();
        }
    }

    /**
     * Returns a dynamic instance of MensajeResource used for entity navigation.
     *
     * @return an instance of MensajeResource
     */
    @Path("{id}/")
    public MensajeResource getMensajeResource(@PathParam("id")
    Integer id) {
        return new MensajeResource(id, context);
    }

    /**
     * Returns all the entities associated with this resource.
     *
     * @return a collection of Mensaje instances
     */
    protected Collection<Mensaje> getEntities(int start, int max) {
        return PersistenceService.getInstance().createQuery("SELECT e FROM Mensaje e").setFirstResult(start).setMaxResults(max).getResultList();
    }

    /**
     * Persist the given entity.
     *
     * @param entity the entity to persist
     */
    protected void createEntity(Mensaje entity) {
        PersistenceService.getInstance().persistEntity(entity);
    }
}
